@@ -36,15 +36,15 @@ const struct option_info options_info[] = {
   { C_L, OPT_LOCKOPT_QUIET,'x', NULL,       no_argument,       "fail silently", NULL },
   { C_RS,OPT_LIMIT_MEM,   'm',  NULL,       required_argument, "set soft DATA, STACK, MEMLOCK and AS limits", "BYTES" },
   { C_RS,OPT_RLIMIT_DATA, 'd',  NULL,       required_argument, "set RLIMIT_DATA", "BYTES" },
-  { C_XS,OPT_RLIMIT_STACK,'s',  NULL,       required_argument, "set RLIMIT_STACK", "BYTES" },
   { C_S, OPT_RLIMIT_MEMLOCK,'l',NULL,       required_argument, "set RLIMIT_MEMLOCK", "BYTES" },
-  { C_XS,OPT_RLIMIT_AS,   'a',  NULL,       required_argument, "set RLIMIT_AS", "BYTES" },
   { C_RS,OPT_RLIMIT_NOFILE,'o', NULL,       required_argument, "set RLIMIT_NOFILE", "FILES" },
   { C_RS,OPT_RLIMIT_NPROC,'p',  NULL,       required_argument, "set RLIMIT_NPROC", "PROCS" },
   { C_RS,OPT_RLIMIT_FSIZE,'f',  NULL,       required_argument, "set RLIMIT_FSIZE", "BYTES" },
   { C_RS,OPT_RLIMIT_CORE, 'c',  NULL,       required_argument, "set RLIMIT_CORE", "BYTES" },
-  { C_XS,OPT_RLIMIT_RSS,  'r',  NULL,       required_argument, "set RLIMIT_RSS", "BYTES" },
   { C_RS,OPT_RLIMIT_CPU,  't',  NULL,       required_argument, "set RLIMIT_CPU", "SECONDS" },
+  { C_XS,OPT_RLIMIT_AS,   'a',  "limit-as",             required_argument, "set RLIMIT_AS", "BYTES" },
+  { C_XS,OPT_RLIMIT_RSS,  'r',  "limit-rss",            required_argument, "set RLIMIT_RSS", "BYTES" },
+  { C_XS,OPT_RLIMIT_STACK,'s',  "limit-stack",          required_argument, "set RLIMIT_STACK", "BYTES" },
   { C_X, OPT_RLIMIT_MEMLOCK,   '\0', "limit-memlock",   required_argument, "set RLIMIT_MEMLOCK", "BYTES" },
   { C_X, OPT_RLIMIT_MSGQUEUE,  '\0', "limit-msgqueue",  required_argument, "set RLIMIT_MSGQUEUE", "BYTES" },
   { C_X, OPT_RLIMIT_NICE,      '\0', "limit-nice",      required_argument, "set RLIMIT_NICE", "NICENESS" },
