@@ -44,8 +44,10 @@ static inline bool usrgrp_specified(const struct sys_entry *entry) {
 
 int usrgrp_parse(struct users_groups *ug, const char *arg);
 int usrgrp_resolve(struct users_groups *ug);
-void usrgrp_resolve_uid(struct users_groups *ug, uid_t nid);
+void usrgrp_resolve_uid(struct users_groups *ug, uid_t nid, bool lookup);
 void usrgrp_print(FILE *out, const char *what, struct users_groups *ug);
 void usrgrp_free(struct users_groups *ug);
+int usrgrp_from_env(struct users_groups *ug,
+                    const char *uid, const char *gid, const char *gidlist);
 
 #endif
