@@ -292,9 +292,9 @@ int main(int argc, char *argv[]) {
   int rc = 0;
   int ret = CHPST_ERROR_CHANGING_STATE;
   int lock_fd = -1;
-  bool in_new_root = false;
   uid_t uid;
   gid_t gid;
+  char *pwd = NULL;
   int fd;
 
   /* As which application were we invoked? */
@@ -318,7 +318,7 @@ int main(int argc, char *argv[]) {
   }
 
   if (!(opt.new_ns & CLONE_NEWNS) &&
-      (set(OPT_NET_NS) || set(OPT_PRIVATE_RUN) || set(OPT_PRIVATE_TMP) ||
+      (set(OPT_NET_NS) || set(OPT_NET_ADOPT) || set(OPT_PRIVATE_RUN) || set(OPT_PRIVATE_TMP) ||
        set(OPT_RO_SYS) || set(OPT_RO_HOME) || set(OPT_RO_ETC) ||
        set(OPT_NEW_ROOT) || set(OPT_PID_NS))) {
     if (is_verbose())
@@ -361,7 +361,9 @@ int main(int argc, char *argv[]) {
   sub_argc = argc - optind;
   if (sub_argc == 0)
     sub_argc++;
-  sub_argv = malloc((sub_argc + 1) * sizeof *sub_argv);
+  if ((sub_argv = malloc((sub_argc + 1) * sizeof *sub_argv)) == NULL)
+    goto finish0;
+
   memcpy(sub_argv, argv + optind, (argc - optind) * sizeof *sub_argv);
   sub_argv[sub_argc] = NULL;
 
@@ -592,9 +594,6 @@ int main(int argc, char *argv[]) {
       if (rc == -1)
         fprintf(stderr, "recursive remounting / as MS_SLAVE: %s", strerror(errno));
     }
-
-    if (opt.new_ns & CLONE_NEWNET)
-      special_mount("/sys", "sysfs", "sysfs", NULL);
   }
 
   if (opt.new_ns & CLONE_NEWUSER) {
@@ -609,30 +608,35 @@ int main(int argc, char *argv[]) {
   }
 
   if (opt.net_adopt) {
-    const char *failed_op = NULL;
-    if ((fd = open(opt.net_adopt, O_RDONLY)) != -1) {
-      if ((rc = setns(fd, CLONE_NEWNET)) == 0) {
-        if ((rc = umount2(opt.net_adopt, MNT_DETACH)) == 0) {
-          if ((rc = unlink(opt.net_adopt)) == -1)
-            failed_op = "unlink";
-        } else failed_op = "umount2";
-      } else {
-        failed_op = "setns"; close(fd);
-      }
-    } else failed_op = "open";
+    const char *failed_op;
+    int old_dir;
+    if ((failed_op = "open .", old_dir = open(".", O_DIRECTORY | O_RDONLY)) == -1)
+      goto adopt_fail0;
+    /* we shouldn't worry about chdir failing if path is absolute */
+    if ((failed_op = "chdir", *opt.net_adopt != '/' && chdir("/var/run/netns")) < 0 ||
+        (failed_op = "open adopted ns", fd = open(opt.net_adopt, O_RDONLY)) < 0)
+      goto adopt_fail1;
+    if ((failed_op = "setns", setns(fd, CLONE_NEWNET)) < 0 ||
+        (failed_op = "umount2", umount2(opt.net_adopt, MNT_DETACH)) < 0 ||
+        (failed_op = "unlink", unlink(opt.net_adopt) < 0))
+      goto adopt_fail2;
+    failed_op = NULL;
+    if (opt.verbosity > 0) fprintf(stderr, "adopted net ns\n");
+adopt_fail2:
+    close(fd);
+    if (fchdir(old_dir) == -1)
+      failed_op = "fchdir to original cwd";
+adopt_fail1:
+    close(old_dir);
+adopt_fail0:
     if (failed_op) {
       perror(failed_op);
       goto finish;
     }
-    if (opt.verbosity > 0) fprintf(stderr, "adopted net ns\n");
   }
 
-  if (set(OPT_NEW_ROOT)) {
-    if (!create_new_root(basename(executable), &pivot))
-      goto finish;
-  }
-
-  set_resource_limits();
+  if (opt.net_adopt || opt.new_ns & CLONE_NEWNET)
+    special_mount("/sys", "sysfs", "sysfs", NULL);
 
   if (set(OPT_FORK_JOIN)) {
     /* Save old signal mask for re-use by child and block all signals
@@ -665,40 +669,21 @@ int main(int argc, char *argv[]) {
    *  Inside child if fork-join used   *
    *************************************/
 
+  pwd = get_current_dir_name();
+  if (pwd == NULL)
+    perror("warning: could not save location of starting directory");
+
   if (set(OPT_NEW_ROOT)) {
-    if (!pivot_to_new_root(&pivot))
+    /* mounting special filesystems after pivoting root will make the kernel
+     * think this is an attempt to bypass a sandboxing program that wants to
+     * hide proc and sys from us. We must mount them while the original mounts
+     * are still visible to prevent a 'Mount too revealing' error in dmesg */
+    if (!create_new_root(basename(executable), &pivot) || !pivot_to_new_root(&pivot))
       goto finish;
-    else
-      in_new_root = true;
-  }
-
-  if (opt.chroot) {
-    rc = chdir(opt.chroot);
-    if (rc == -1) {
-      perror("chdir for chroot");
-      goto finish;
-    }
-    rc = chroot(".");
-    if (rc == -1) {
-      perror("chroot");
-      goto finish;
-    }
-    if (is_verbose())
-      fprintf(stderr, "entered chroot: %s\n", opt.chroot);
-  }
-
-  if (opt.chdir) {
-    rc = chdir(opt.chdir);
-    if (rc == -1) {
-      perror("chdir");
-      goto finish;
-    }
-    if (is_verbose())
-      fprintf(stderr, "change directory: %s\n", opt.chdir);
-  }
-
-  if (opt.new_ns & CLONE_NEWPID) {
-    special_mount("/proc", "proc", "procfs", NULL);
+  } else {
+    /* we can un+remount without much issue */
+    if (opt.new_ns & CLONE_NEWPID)
+      special_mount("/proc", "proc", "procfs", NULL);
   }
 
   if (set(OPT_PRIVATE_RUN) &&
@@ -730,6 +715,40 @@ int main(int argc, char *argv[]) {
       (remount_ro("/etc") == -1))
     goto finish;
 
+  if ((set(OPT_NEW_ROOT) ||
+       set(OPT_PRIVATE_RUN) ||
+       set(OPT_PRIVATE_TMP) ||
+       set(OPT_PROTECT_HOME) ||
+       set(OPT_RO_SYS) ||
+       set(OPT_RO_ETC)) &&
+      (pwd != NULL && chdir(pwd) == -1))
+    perror("warning: could not change to original directory relative to new fs");
+
+  if (opt.chroot) {
+    rc = chdir(opt.chroot);
+    if (rc == -1) {
+      perror("chdir for chroot");
+      goto finish;
+    }
+    rc = chroot(".");
+    if (rc == -1) {
+      perror("chroot");
+      goto finish;
+    }
+    if (is_verbose())
+      fprintf(stderr, "entered chroot: %s\n", opt.chroot);
+  }
+
+  if (opt.chdir) {
+    rc = chdir(opt.chdir);
+    if (rc == -1) {
+      perror("chdir");
+      goto finish;
+    }
+    if (is_verbose())
+      fprintf(stderr, "change directory: %s\n", opt.chdir);
+  }
+
   if (set(OPT_SETUIDGID) &&
       (opt.new_ns & CLONE_NEWUSER) == 0 &&
       !drop_user(&opt.users_groups))
@@ -749,6 +768,8 @@ int main(int argc, char *argv[]) {
 
   pivot_tidy(&pivot);
 
+  set_resource_limits();
+
   /* Launch the target */
   rc = execvp(executable, sub_argv);
 
@@ -767,10 +788,7 @@ finish:
      3) not be necessary when --fork-join is not used.
    */
 
-  if (pivot.path && !in_new_root) {
-    free_rootfs_data();
-  }
-
+  free_rootfs_data();
   pivot_tidy(&pivot);
   if (run_dir_fd != -1)
     close(run_dir_fd);
@@ -778,6 +796,7 @@ finish:
   if (lock_fd != -1)
     close(lock_fd);
 
+  free(pwd);
   free(sub_argv);
   free(run_dir);
   free(extra_env.gidlist);
